from django.http import JsonResponse
from django.views import View
from django.views.generic.list import BaseListView
from django.db.models import Q
from django.core.serializers import serialize
import json


class AutocompleteView(View):
    """Base view for handling autocomplete requests."""
    
    def get_results(self, query):
        """
        Override this method to return autocomplete results.
        Should return a list of strings or dictionaries.
        """
        raise NotImplementedError("Subclasses must implement get_results()")
    
    def get(self, request, *args, **kwargs):
        query = request.GET.get('q', '')
        
        if not query:
            return JsonResponse({'results': [], 'query': query})
        
        results = self.get_results(query)
        return JsonResponse({'results': results, 'query': query})


class ModelAutocompleteView(AutocompleteView):
    """
    Base view for model-based autocomplete.
    Subclasses should set model and search_fields attributes.
    """
    model = None
    search_fields = []
    limit = 20
    
    def get_queryset(self):
        """Get the base queryset. Override to add custom filtering."""
        if self.model is None:
            raise NotImplementedError("ModelAutocompleteView requires a model attribute")
        return self.model.objects.all()
    
    def get_search_fields(self):
        """Get the fields to search in. Override to customize."""
        if not self.search_fields:
            raise NotImplementedError("ModelAutocompleteView requires search_fields attribute")
        return self.search_fields
    
    def format_result(self, obj):
        """
        Format a model instance for the autocomplete response.
        Returns a dict with 'value' (obj.id) and 'label' (first search field).
        Override to customize the output format.
        """
        search_fields = self.get_search_fields()
        
        # Get the value for the label from the first search field
        if search_fields:
            label_field = search_fields[0]
            label_value = getattr(obj, label_field, str(obj))
        else:
            label_value = str(obj)
        
        return {
            'value': str(obj.id),
            'label': str(label_value),
        }
    
    def get_results(self, query):
        """Search the model and return results."""
        queryset = self.get_queryset()
        search_fields = self.get_search_fields()
        
        # Build the Q object for searching
        search_query = Q()
        for field in search_fields:
            search_query |= Q(**{f"{field}__icontains": query})
        
        # Filter and limit results
        results = queryset.filter(search_query)[:self.limit]
        
        # Format results
        return [self.format_result(obj) for obj in results]


class SimpleAutocompleteView(AutocompleteView):
    """
    Simple autocomplete view that returns static choices.
    Useful for non-model based autocomplete.
    """
    choices = []
    
    def get_choices(self):
        """Get the list of choices. Override to make dynamic."""
        return self.choices
    
    def get_results(self, query):
        """Filter choices based on query."""
        choices = self.get_choices()
        query_lower = query.lower()
        
        return [
            choice for choice in choices 
            if query_lower in str(choice).lower()
        ][:20]  # Limit to 20 results


"""
Example implementations of autocomplete views and forms.
These can be used as references or directly in your project.
"""

from django import forms
from django.contrib.auth.models import User
from .views import ModelAutocompleteView, SimpleAutocompleteView
from .fields import AutocompleteField, ModelAutocompleteField
from .widgets import AutocompleteWidget


# Example Views

class UserAutocompleteView(ModelAutocompleteView):
    """Example autocomplete view for Django User model."""
    model = User
    search_fields = ['username', 'email', 'first_name', 'last_name']
    
    def format_result(self, obj):
        """Return user display format."""
        if obj.get_full_name():
            return f"{obj.username} ({obj.get_full_name()})"
        return obj.username


class FruitAutocompleteView(SimpleAutocompleteView):
    """Example static autocomplete view."""
    choices = [
        'Apple', 'Banana', 'Cherry', 'Date', 'Elderberry',
        'Fig', 'Grape', 'Honeydew', 'Indian Fig', 'Jackfruit',
        'Kiwi', 'Lemon', 'Mango', 'Nectarine', 'Orange',
        'Papaya', 'Quince', 'Raspberry', 'Strawberry', 'Tangerine',
        'Ugli Fruit', 'Valencia Orange', 'Watermelon', 'Xigua', 'Yellow Passion Fruit', 'Zucchini'
    ]


class CountryAutocompleteView(SimpleAutocompleteView):
    """Example autocomplete with more complex data."""
    
    def get_choices(self):
        # In a real app, this might come from a database or API
        return [
            {'code': 'US', 'name': 'United States'},
            {'code': 'CA', 'name': 'Canada'},
            {'code': 'MX', 'name': 'Mexico'},
            {'code': 'GB', 'name': 'United Kingdom'},
            {'code': 'FR', 'name': 'France'},
            {'code': 'DE', 'name': 'Germany'},
            {'code': 'IT', 'name': 'Italy'},
            {'code': 'ES', 'name': 'Spain'},
            {'code': 'JP', 'name': 'Japan'},
            {'code': 'CN', 'name': 'China'},
            {'code': 'IN', 'name': 'India'},
            {'code': 'BR', 'name': 'Brazil'},
            {'code': 'AU', 'name': 'Australia'},
        ]
    
    def get_results(self, query):
        """Search in both code and name."""
        choices = self.get_choices()
        query_lower = query.lower()
        
        results = []
        for country in choices:
            if (query_lower in country['code'].lower() or 
                query_lower in country['name'].lower()):
                results.append(country)
        
        return results[:10]


# Example Forms

class ExampleForm(forms.Form):
    """Example form using different autocomplete fields."""
    
    # Simple text autocomplete
    fruit = AutocompleteField(
        url='/autocomplete/fruits/',
        help_text='Start typing to search for fruits'
    )
    
    # Model-based autocomplete
    user = ModelAutocompleteField(
        queryset=User.objects.all(),
        url='/autocomplete/users/',
        required=False,
        help_text='Search for users by username, email, or name'
    )
    
    # Custom widget on a regular field
    country = forms.CharField(
        widget=AutocompleteWidget(url='/autocomplete/countries/'),
        help_text='Search for countries'
    )


class UserSelectionForm(forms.Form):
    """Example form for user selection with autocomplete."""
    
    assigned_to = ModelAutocompleteField(
        queryset=User.objects.filter(is_active=True),
        url='/autocomplete/active-users/',
        label='Assign to User',
        help_text='Search by username, email, or name'
    )
    
    reviewers = forms.CharField(
        widget=AutocompleteWidget(url='/autocomplete/users/'),
        help_text='Enter multiple reviewers separated by commas',
        required=False
    )


# Example URLs configuration
"""
# In your urls.py:

from django.urls import path
from suitable_django_autocomplete.examples import (
    UserAutocompleteView,
    FruitAutocompleteView,
    CountryAutocompleteView
)

urlpatterns = [
    path('autocomplete/users/', UserAutocompleteView.as_view(), name='autocomplete-users'),
    path('autocomplete/fruits/', FruitAutocompleteView.as_view(), name='autocomplete-fruits'),
    path('autocomplete/countries/', CountryAutocompleteView.as_view(), name='autocomplete-countries'),
]
"""


from django import forms
from .widgets import AutocompleteWidget


class AutocompleteField(forms.CharField):
    """Form field that uses the AutocompleteWidget by default."""
    
    def __init__(self, *args, url=None, min_length=2, debounce_delay=300, attrs=None, 
                 host_attrs=None, **kwargs):
        self.url = url
        self.min_length = min_length
        self.debounce_delay = debounce_delay
        self.host_attrs = host_attrs
        
        # Set the widget if not already specified
        if 'widget' not in kwargs:
            kwargs['widget'] = AutocompleteWidget(
                url=url,
                min_length=min_length,
                debounce_delay=debounce_delay,
                attrs=attrs,
                host_attrs=host_attrs
            )
        
        super().__init__(*args, **kwargs)


class ModelAutocompleteField(forms.ModelChoiceField):
    """Model choice field that uses the AutocompleteWidget."""
    
    def __init__(self, queryset, *args, url=None, min_length=2, debounce_delay=300, attrs=None, 
                 host_attrs=None, search_fields=None, **kwargs):
        self.url = url
        self.min_length = min_length
        self.debounce_delay = debounce_delay
        self.host_attrs = host_attrs
        self.search_fields = search_fields or []
        
        # Set the widget if not already specified
        if 'widget' not in kwargs:
            kwargs['widget'] = AutocompleteWidget(
                url=url,
                min_length=min_length,
                debounce_delay=debounce_delay,
                attrs=attrs,
                host_attrs=host_attrs
            )
        
        super().__init__(queryset, *args, **kwargs)
    
    
    def to_python(self, value):
        """Convert the autocomplete value to a model instance."""
        if value in self.empty_values:
            return None
        try:
            # Try to get by primary key first
            key = self.to_field_name or 'pk'
            value = self.queryset.get(**{key: value})
        except (ValueError, TypeError, self.queryset.model.DoesNotExist):
            # If that fails, try to get by string representation
            # This allows for more flexible autocomplete implementations
            try:
                for obj in self.queryset:
                    if str(obj) == value:
                        return obj
            except:
                pass
            raise forms.ValidationError(
                self.error_messages['invalid_choice'],
                code='invalid_choice',
                params={'value': value},
            )
        return value
    
    def get_display_value(self, obj):
        """
        Get a display value for the given model instance.
        Uses the first search field if available, otherwise str(obj).
        """
        if not obj:
            return ''
            
        # If search_fields are specified, use the first one
        if self.search_fields:
            first_field = self.search_fields[0]
            # Handle related fields (e.g., 'user__username')
            if '__' in first_field:
                parts = first_field.split('__')
                value = obj
                for part in parts:
                    value = getattr(value, part, None)
                    if value is None:
                        break
                return str(value) if value is not None else str(obj)
            else:
                return str(getattr(obj, first_field, obj))
        
        # Fall back to string representation
        return str(obj)
    
    def prepare_value(self, value):
        """
        Prepare the value for display in the widget.
        Also sets the initial_display_value for model instances.
        """
        # Let the parent class handle the basic value preparation
        prepared_value = super().prepare_value(value)
        
        # Set the initial_display_value if we have a model instance and no manual value
        if value and not self.widget.initial_display_value:
            try:
                if hasattr(value, 'pk'):
                    # It's already a model instance
                    obj = value
                elif prepared_value:
                    # It's an ID, try to fetch the instance
                    obj = self.queryset.get(pk=prepared_value)
                else:
                    obj = None
                
                if obj:
                    # Use the widget's helper method
                    self.widget.set_initial_display_value_from_instance(obj, self.search_fields)
            except (ValueError, TypeError, self.queryset.model.DoesNotExist):
                pass
        
        # Ensure we return a string
        return str(prepared_value) if prepared_value is not None else ''


"""Modern Django autocomplete widget using web components."""

__version__ = "0.1.0"

from .widgets import AutocompleteWidget
from .fields import AutocompleteField, ModelAutocompleteField
from .views import AutocompleteView, ModelAutocompleteView, SimpleAutocompleteView

__all__ = [
    "AutocompleteWidget",
    "AutocompleteField", 
    "ModelAutocompleteField",
    "AutocompleteView",
    "ModelAutocompleteView",
    "SimpleAutocompleteView",
]


from django import forms
from django.urls import reverse_lazy


class AutocompleteWidget(forms.TextInput):
    """A suitable autocomplete widget for Django forms using web components."""

    template_name = "suitable_django_autocomplete/autocomplete.html"

    def __init__(self, url=None, attrs=None, min_length=2, debounce_delay=300,
                 value_field='value', label_field='label', initial_display_value=None,
                 host_attrs=None):
        self.url = url
        self.min_length = min_length
        self.debounce_delay = debounce_delay
        self.value_field = value_field
        self.label_field = label_field
        self.initial_display_value = initial_display_value
        self.host_attrs = host_attrs or {}
        super().__init__(attrs)

    def get_context(self, name, value, attrs):
        context = super().get_context(name, value, attrs)
        context["widget"].update(
            {
                "url": self.url or reverse_lazy("autocomplete"),
                "min_length": self.min_length,
                "debounce_delay": self.debounce_delay,
                "value_field": self.value_field,
                "label_field": self.label_field,
                "initial_display_value": self.initial_display_value,
                "host_attrs": self.host_attrs,
            }
        )
        return context
    
    def set_initial_display_value_from_instance(self, obj, search_fields=None):
        """
        Helper method to set initial_display_value from a model instance.
        This can be called by ModelAutocompleteField.
        """
        if not obj:
            return
            
        # Use search_fields if provided to determine display value
        if search_fields:
            first_field = search_fields[0]
            # Handle related fields (e.g., 'user__username')
            if '__' in first_field:
                parts = first_field.split('__')
                value = obj
                for part in parts:
                    value = getattr(value, part, None)
                    if value is None:
                        break
                self.initial_display_value = str(value) if value is not None else str(obj)
            else:
                self.initial_display_value = str(getattr(obj, first_field, obj))
        else:
            # Fall back to string representation
            self.initial_display_value = str(obj)

    # When migrating to 5.2 only, use this to module load instead of in template. Nice.
    # class Media:
    #     js = [
    #         Script(
    #             "suitable_django_autocomplete/autocomplete.js",
    #             **{
    #                 "type": "module",
    #             },
    #         ),
    #     ]
